# -*- coding: utf-8 -*-
from unittest import TestCase

from telephonie.core.eventtypes import Event


class TestEvent(TestCase):
    EVENT_COMMAND_REPLY = "Content-Type: command/reply\nReply-Text: +OK accepted\n\n"
    EVENT_AUTH_REQUEST = "Content-Type: auth/request\n\n"
    EVENT_CONTENT_LENGTH = "Content-Length: 491\nContent-Type: text/event-plain\n\n"
    EVENT_PLAIN = """Event-Name: RE_SCHEDULE
Core-UUID: 12640749-db62-421c-beac-4863eac76510
FreeSWITCH-Hostname: vocaldev
FreeSWITCH-IPv4: 10.0.0.108
FreeSWITCH-IPv6: %3A%3A1
Event-Date-Local: 2011-01-03%2018%3A33%3A56
Event-Date-GMT: Mon,%2003%20Jan%202011%2017%3A33%3A56%20GMT
Event-Date-Timestamp: 1294076036427219
Event-Calling-File: switch_scheduler.c
Event-Calling-Function: switch_scheduler_execute
Event-Calling-Line-Number: 65
Task-ID: 1
Task-Desc: heartbeat
Task-Group: core
Task-Runtime: 1294076056

"""




    def test_command_reply(self):
        ev = Event(self.EVENT_COMMAND_REPLY)
        self.assertEquals(ev.get_content_type(), "command/reply") 
        self.assertEquals(ev.get_reply_text(), "+OK accepted") 
        self.assertTrue(ev.is_reply_text_success())

    def test_auth_request(self):
        ev = Event(self.EVENT_AUTH_REQUEST)
        self.assertEquals(ev.get_content_type(), "auth/request")

    def test_event_plain(self):
        ev1 = Event(self.EVENT_CONTENT_LENGTH)
        self.assertEquals(ev1.get_content_length(), 491)
        self.assertEquals(ev1.get_content_type(), "text/event-plain")
        ev2 = Event(self.EVENT_PLAIN)
        self.assertEquals(ev2.get_header("Event-Name"), "RE_SCHEDULE")
        self.assertEquals(len(self.EVENT_PLAIN), ev1.get_content_length())




# -*- coding: utf-8 -*-
"""
Event Types classes
"""

from urllib import unquote
from urllib import quote


class Event(object):
    '''Event class'''
    def __init__(self, buffer=""):
        self._headers = {}
        self._raw_body = ''
        self._raw_headers = ''
        self._u_raw_headers = ''
        if buffer:
            # Sets event headers from buffer.
            for line in buffer.splitlines():
                try:
                    var, val = line.rstrip().split(': ', 1)
                    self.set_header(var, val)
                except ValueError: 
                    pass

    def __getitem__(self, key):
        return self.get_header(key)

    def __setitem__(self, key, value):
        self.set_header(key, value)

    def get_content_length(self):
        '''
        Gets Content-Length header as integer.

        Returns 0 If length not found.
        '''
        length = self.get_header('Content-Length')
        if length:
            try:
                return int(length)
            except:
                return 0
        return 0

    def get_reply_text(self):
        '''
        Gets Reply-Text header as string.

        Returns None if header not found.
        '''
        return self.get_header('Reply-Text') 

    def is_reply_text_success(self):
        '''
        Returns True if ReplyText header begins with +OK.
        
        Returns False otherwise.
        '''
        reply = self.get_reply_text()
        return reply and reply[:3] == '+OK'

    def get_content_type(self):
        '''
        Gets Content-Type header as string.

        Returns None if header not found.
        '''
        return self.get_header('Content-Type')

    def get_headers(self):
        '''
        Gets all headers as a python dict.
        '''
        return self._headers

    def set_headers(self, headers):
        '''
        Sets all headers from dict.
        '''
        self._headers = headers.copy()

    def get_header(self, key, defaultvalue=None):
        '''
        Gets a specific header as string.

        Returns None if header not found.
        '''
        try:
            return self._headers[key]
        except KeyError:
            return defaultvalue

    def set_header(self, key, value):
        '''
        Sets a specific header.
        '''
        key = key.strip()
        value = value.strip()
        u_value = unquote(value)
        self._raw_headers += "%s: %s\n" % (key, value)
        self._u_raw_headers += "%s: %s\n" % (key, u_value)
        self._headers[key] = u_value

    def get_body(self):
        '''
        Gets raw Event body.
        '''
        return self._raw_body

    def set_body(self, data):
        '''
        Sets raw Event body.
        '''
        self._raw_body = data

    def get_raw_headers(self):
        '''
        Gets raw headers (quoted).
        '''
        return self._raw_headers

    def get_unquoted_raw_headers(self):
        '''
        Gets raw headers (unquoted).
        '''
        return self._u_raw_headers

    def get_raw_event(self):
        '''
        Gets raw Event (quoted).
        '''
        return self._raw_headers + self._raw_body + '\n'

    def get_unquoted_raw_event(self):
        '''
        Gets raw Event (unquoted).
        '''
        return self._u_raw_headers + self._raw_body + '\n'

    def __str__(self):
        return '<%s headers=%s, body=%s>' \
               % (self.__class__.__name__, 
                  str(self.get_unquoted_raw_headers().replace('\n', '\\n')), 
                  str(self.get_body()).replace('\n', '\\n'))


class ApiResponse(Event):
    def __init__(self, buffer=""):
        Event.__init__(self, buffer)

    @classmethod
    def cast(self, event):
        '''
        Makes an ApiResponse instance from Event instance.
        '''
        cls = ApiResponse(event.get_raw_headers())
        cls.set_body(event.get_body())
        return cls

    def get_response(self):
        '''
        Gets response for api command.
        '''
        return self.get_body().strip()

    def is_success(self):
        '''
        Returns True if api command is a success.
        
        Otherwise returns False.
        '''
        return self._raw_body and self._raw_body[:3] == '+OK'


class BgapiResponse(Event):
    def __init__(self, buffer=""):
        Event.__init__(self, buffer)

    @classmethod
    def cast(self, event):
        '''
        Makes a BgapiResponse instance from Event instance.
        '''
        cls = BgapiResponse(event.get_raw_headers())
        cls.set_body(event.get_body())
        return cls

    def get_response(self):
        '''
        Gets response for bgapi command.
        '''
        return self.get_reply_text()

    def get_job_uuid(self):
        '''
        Gets Job-UUID from bgapi command.
        '''
        return self.get_header('Job-UUID')

    def is_success(self):
        '''
        Returns True if bgapi command is a success.
        
        Otherwise returns False.
        '''
        return self.is_reply_text_success()


class CommandResponse(Event):
    def __init__(self, buffer=""):
        Event.__init__(self, buffer)

    @classmethod
    def cast(self, event):
        '''
        Makes a CommandResponse instance from Event instance.
        '''
        cls = CommandResponse(event.get_raw_headers())
        cls.set_body(event.get_body())
        return cls

    def get_response(self):
        '''
        Gets response for a command.
        '''
        return self.get_reply_text()

    def is_success(self):
        '''
        Returns True if command is a success.
        
        Otherwise returns False.
        '''
        return self.is_reply_text_success()






# -*- coding: utf-8 -*-
"""
Event Socket class
"""

from __future__ import with_statement
import types
import string
import gevent
import gevent.socket as socket
import gevent.queue as queue
import gevent.pool
from gevent import GreenletExit
from gevent.coros import RLock
from telephonie.core.commands import Commands
from telephonie.core.eventtypes import Event
from telephonie.core.eventtypes import (CommandResponse, ApiResponse, BgapiResponse)
from telephonie.core.errors import (LimitExceededError, ConnectError)


EOL = "\n"
MAXLINES_PER_EVENT = 2000



class EventSocket(Commands):
    '''EventSocket class'''
    def __init__(self, filter="ALL", pool_size=500):
        # Callbacks for reading events and sending responses.
        self._response_callbacks = {'api/response':self._api_response,
                                    'command/reply':self._command_reply,
                                    'text/event-plain':self._event_plain,
                                    'auth/request':self._auth_request,
                                    'text/disconnect-notice':self._disconnect_notice
                                   }
        # Default event filter.
        self._filter = filter
        # Synchronized Gevent based Queue for response events.
        self._response_queue = queue.Queue(1)
        # Lock to force eventsocket commands to be sequential.
        self._lock = RLock()
        # Sets connected to False.
        self.connected = False
        # Creates pool for spawning if pool_size > 0
        if pool_size > 0:
            self.pool = gevent.pool.Pool(pool_size)
        else:
            self.pool = None
        # Handler thread
        self._handler_thread = None

    def _spawn(self, func, *args, **kwargs):
        '''
        Spawns with or without pool.
        '''
        if self.pool:
            self.pool.spawn(func, *args, **kwargs)
        else:
            gevent.spawn(func, *args, **kwargs)

    def is_connected(self):
        '''
        Checks if connected and authenticated to eventsocket.

        Returns True or False.
        '''
        return self.connected

    def start_event_handler(self):
        '''
        Starts Event handler in background.
        '''
        self._handler_thread = gevent.spawn(self.handle_events)

    def stop_event_handler(self):
        '''
        Stops Event handler.
        '''
        if self._handler_thread:
            self._handler_thread.kill()
        
    def handle_events(self):
        '''
        Gets and Dispatches events in an endless loop using gevent spawn.
        '''
        while True:
            try:
                # Gets event and dispatches to handler.
                ev = self.get_event()
                # Only dispatches event if Event-Name header found.
                if ev and ev.get_header('Event-Name'):
                    self._spawn(self.dispatch_event, ev)
                    gevent.sleep(0.005)
            except (LimitExceededError, ConnectError, socket.error):
                self.connected = False
                return
            except GreenletExit, e:
                self.connected = False
                return
        
    def read_event(self):
        '''
        Reads one Event from socket until EOL.

        Returns Event instance.

        Raises LimitExceededError if MAXLINES_PER_EVENT is reached.
        '''
        buff = ''
        for x in range(MAXLINES_PER_EVENT):
            line = self.transport.read_line()
            if line == '':
                raise ConnectError("connection closed")
            elif line == EOL:
                # When matches EOL, creates Event and returns it.
                return Event(buff)
            else:
                # Else appends line to current buffer.
                buff += line
        raise LimitExceededError("max lines per event (%d) reached" % MAXLINES_PER_EVENT)

    def read_raw(self, event):
        '''
        Reads raw data based on Event Content-Length.

        Returns raw string or None if not found.
        '''
        length = event.get_content_length()
        # Reads length bytes if length > 0
        if length:
            return self.transport.read(int(length))
        return None

    def read_raw_response(self, event, raw):
        '''
        Extracts raw response from raw buffer and length based on Event Content-Length.

        Returns raw string or None if not found.
        '''
        length = event.get_content_length()
        if length:
            return raw[-length:]
        return None

    def get_event(self):
        '''
        Gets complete Event, and processes response callback.
        '''
        event = self.read_event()
        # Gets callback response for this event (sets to self._unknown_event, if no matching callable)
        _get_response = self._response_callbacks.get(event.get_content_type(), self._unknown_event)
        # If callback response found, starts this method to get final event.
        if _get_response:
            event = _get_response(event)
        return event

    def _auth_request(self, event):
        '''
        Receives auth/request callback.
        '''
        # Pushes Event to response events queue and returns Event.
        self._response_queue.put(event)
        return event

    def _api_response(self, event):
        '''
        Receives api/response callback.
        '''
        # Gets raw data for this event.
        raw = self.read_raw(event)
        # If raw was found, this is our Event body.
        if raw:
            event.set_body(raw)
        # Pushes Event to response events queue and returns Event.
        self._response_queue.put(event)
        return event

    def _command_reply(self, event):
        '''
        Receives command/reply callback.
        '''
        # Pushes Event to response events queue and returns Event.
        self._response_queue.put(event)
        return event

    def _event_plain(self, event):
        '''
        Receives text/event-plain callback.
        '''
        # Gets raw data for this event
        raw = self.read_raw(event)
        # If raw was found drops current event 
        # and replaces with Event created from raw
        if raw:
            event = Event(raw)
            # Gets raw response from Event Content-Length header 
            # and raw buffer
            raw_response = self.read_raw_response(event, raw)
            # If rawresponse was found, this is our Event body
            if raw_response:
                event.set_body(raw_response)
        # Returns Event
        return event

    def _disconnect_notice(self, event):
        '''
        Receives text/disconnect-notice callback.
        '''
        self.disconnect()

    def _unknown_event(self, event):
        '''
        Receives unkown event type Callbacks.

        Can be implemented in subclass to process unknown event types.
        '''
        pass

    def dispatch_event(self, event):
        '''
        Dispatches one event with callback.
        
        E.g. Receives Background_Job event and calls on_background_job function.
        '''
        method = 'on_' + event.get_header('Event-Name').lower()
        callback = getattr(self, method, None)
        # When no callbacks found, call unbound_event.
        if not callback:
            callback = self.unbound_event(event)
        # Calls callback.
        # On exception calls callback_failure method.
        try: 
            callback(event)
        except: 
            self.callback_failure(event)
    
    def unbound_event(self, event):
        '''
        Catches all unbound events from FreeSWITCH.
        
        Can be implemented by the subclass.
        '''
        pass
    
    def callback_failure(self, event):
        '''
        Called when callback to an event fails.
        
        Can be implemented by the subclass.
        '''
        pass
    
    def disconnect(self):
        '''
        Disconnects from eventsocket and stops handling events.
        '''
        self.transport.close()
        self.stop_event_handler()
        self.connected = False

    def connect(self):
        '''
        Connects to eventsocket.

        Must be implemented by subclass.
        '''
        pass

    def _send(self, cmd):
        if isinstance(cmd, types.UnicodeType):
            cmd = cmd.encode("utf-8")
        self.transport.write(cmd + EOL*2)
        
    def _sendmsg(self, name, arg=None, uuid="", lock=False, loops=1):
        if isinstance(name, types.UnicodeType):
            name = name.encode("utf-8")
        if isinstance(arg, types.UnicodeType):
            arg = arg.encode("utf-8")
        msg = "sendmsg %s\ncall-command: execute\n" % uuid
        msg += "execute-app-name: %s\n" % name
        if lock is True:
            msg += "event-lock: true\n"
        if loops > 1:
            msg += "loops: %d\n" % loops
        if arg:
            arglen = len(arg)
            msg += "content-type: text/plain\ncontent-length: %d\n\n%s\n" % (arglen, arg)
        self.transport.write(msg + EOL)
        
    def _protocol_send(self, command, args=""):
        with self._lock:
            self._send("%s %s" % (command, args))
            event = self._response_queue.get()
        # Casts Event to appropriate event type :
        # Casts to ApiResponse, if event is api
        if command == 'api':
            event = ApiResponse.cast(event)
        # Casts to BgapiResponse, if event is bgapi 
        elif command == "bgapi":
            event = BgapiResponse.cast(event)
        # Casts to CommandResponse by default
        else:
            event = CommandResponse.cast(event)
        return event
    
    def _protocol_sendmsg(self, name, args=None, uuid="", lock=False, loops=1):
        with self._lock:
            self._sendmsg(name, args, uuid, lock, loops)
            event = self._response_queue.get()
        # Always casts Event to CommandResponse
        return CommandResponse.cast(event)




# -*- coding: utf-8 -*-
"""
Log classes : stdout, syslog and file loggers
"""

import gevent.monkey
gevent.monkey.patch_thread()

import logging
import logging.handlers
from logging import RootLogger
import sys
import os

LOG_DEBUG = logging.DEBUG
LOG_ERROR = logging.ERROR
LOG_INFO = logging.INFO
LOG_WARN = logging.WARN
LOG_WARNING = logging.WARNING
LOG_CRITICAL = logging.CRITICAL
LOG_FATAL = logging.FATAL
LOG_NOTSET = logging.NOTSET


__default_servicename__ = os.path.splitext(os.path.basename(sys.argv[0]))[0]


class StdoutLogger(object):
    def __init__(self, loglevel=LOG_DEBUG, servicename=__default_servicename__):
        self.loglevel = loglevel
        h = logging.StreamHandler()
        h.setLevel(loglevel)
        fmt = logging.Formatter("%(asctime)s "+servicename+"[%(process)d]: %(levelname)s: %(message)s")
        h.setFormatter(fmt)
        self._logger = RootLogger(loglevel)
        self._logger.addHandler(h)

    def set_debug(self):
        self.loglevel = LOG_DEBUG
        self._logger.setLevel(self.loglevel)
        
    def set_info(self):
        self.loglevel = LOG_INFO
        self._logger.setLevel(self.loglevel)

    def info(self, msg):
        self._logger.info(str(msg))

    def debug(self, msg):
        self._logger.debug(str(msg))

    def warn(self, msg):
        self._logger.warn(str(msg))

    def error(self, msg):
        self._logger.error(str(msg))

    def write(self, msg):
        self.info(msg)


class Syslog(logging.handlers.SysLogHandler):
    LOG_EMERG     = 0       #  system is unusable
    LOG_ALERT     = 1       #  action must be taken immediately
    LOG_CRIT      = 2       #  critical conditions
    LOG_ERR       = 3       #  error conditions
    LOG_WARNING   = 4       #  warning conditions
    LOG_NOTICE    = 5       #  normal but significant condition
    LOG_INFO      = 6       #  informational
    LOG_DEBUG     = 7       #  debug-level messages


    #  facility codes
    LOG_KERN      = 0       #  kernel messages
    LOG_USER      = 1       #  random user-level messages
    LOG_MAIL      = 2       #  mail system
    LOG_DAEMON    = 3       #  system daemons
    LOG_AUTH      = 4       #  security/authorization messages
    LOG_SYSLOG    = 5       #  messages generated internally by syslogd
    LOG_LPR       = 6       #  line printer subsystem
    LOG_NEWS      = 7       #  network news subsystem
    LOG_UUCP      = 8       #  UUCP subsystem
    LOG_CRON      = 9       #  clock daemon
    LOG_AUTHPRIV  = 10  #  security/authorization messages (private)

    #  other codes through 15 reserved for system use
    LOG_LOCAL0    = 16      #  reserved for local use
    LOG_LOCAL1    = 17      #  reserved for local use
    LOG_LOCAL2    = 18      #  reserved for local use
    LOG_LOCAL3    = 19      #  reserved for local use
    LOG_LOCAL4    = 20      #  reserved for local use
    LOG_LOCAL5    = 21      #  reserved for local use
    LOG_LOCAL6    = 22      #  reserved for local use
    LOG_LOCAL7    = 23      #  reserved for local use


    priority_names = {
        "alert":    LOG_ALERT,
        "crit":     LOG_CRIT,
        "critical": LOG_CRIT,
        "debug":    LOG_DEBUG,
        "emerg":    LOG_EMERG,
        "err":      LOG_ERR,
        "error":    LOG_ERR,        #  DEPRECATED
        "info":     LOG_INFO,
        "notice":   LOG_NOTICE,
        "panic":    LOG_EMERG,      #  DEPRECATED
        "notice":   LOG_NOTICE,
        "warn":     LOG_WARNING,    #  DEPRECATED
        "warning":  LOG_WARNING,
        "info_srv":  LOG_INFO,
        "error_srv":  LOG_ERR,
        "debug_srv":  LOG_DEBUG,
        "warn_srv":  LOG_WARNING,
        }

    facility_names = {
        "auth":     LOG_AUTH,
        "authpriv": LOG_AUTHPRIV,
        "cron":     LOG_CRON,
        "daemon":   LOG_DAEMON,
        "kern":     LOG_KERN,
        "lpr":      LOG_LPR,
        "mail":     LOG_MAIL,
        "news":     LOG_NEWS,
        "security": LOG_AUTH,       #  DEPRECATED
        "syslog":   LOG_SYSLOG,
        "user":     LOG_USER,
        "uucp":     LOG_UUCP,
        "local0":   LOG_LOCAL0,
        "local1":   LOG_LOCAL1,
        "local2":   LOG_LOCAL2,
        "local3":   LOG_LOCAL3,
        "local4":   LOG_LOCAL4,
        "local5":   LOG_LOCAL5,
        "local6":   LOG_LOCAL6,
        "local7":   LOG_LOCAL7,
        }


class SysLogger(StdoutLogger):
    def __init__(self, addr='/dev/log', syslogfacility="local0", \
                 loglevel=LOG_DEBUG, servicename=__default_servicename__):
        fac = Syslog.facility_names[syslogfacility]
        h = Syslog(address=addr, facility=fac)
        h.setLevel(loglevel)
        fmt = logging.Formatter(servicename+"[%(process)d]: %(levelname)s: %(message)s")
        h.setFormatter(fmt)
        self._logger = RootLogger(loglevel)
        self._logger.addHandler(h)


class FileLogger(StdoutLogger):
    def __init__(self, logfile='/tmp/%s.log' % __default_servicename__, \
                 loglevel=LOG_DEBUG, servicename=__default_servicename__):
        h = logging.FileHandler(filename=logfile)
        h.setLevel(loglevel)
        fmt = logging.Formatter("%(asctime)s "+servicename+"[%(process)d]: %(levelname)s: %(message)s")
        h.setFormatter(fmt)
        self._logger = RootLogger(loglevel)
        self._logger.addHandler(h)




# -*- coding: utf-8 -*-
"""
Exceptions classes
"""

class LimitExceededError(Exception): 
    '''Exception class when MAXLINES_PER_EVENT is reached'''
    pass


class ConnectError(Exception): 
    '''Exception class for connection'''
    pass




# -*- coding: utf-8 -*-
"""
Telephonie  - Framework to create telephony applications using FreeSWITCH 

Primary Authors - Michael Ricordeau (tamiel) and Miglu (miglu)

Telephonie is inspired by and uses code from - https://github.com/fiorix/eventsocket
"""

__name__ = "telephonie"

__all__ = ['core', 'utils']




# -*- coding: utf-8 -*-
"""
Outbound Event Socket class
"""

from gevent.server import StreamServer
from gevent.timeout import Timeout
from telephonie.core.eventsocket import EventSocket
from telephonie.core.transport import OutboundTransport
from telephonie.core.errors import ConnectError



class OutboundEventSocket(EventSocket):
    '''
    FreeSWITCH Outbound Event Socket.
    
    A new instance of this class is created for every call/ session from FreeSWITCH.
    '''
    def __init__(self, socket, address, filter="ALL", pool_size=500, connect_timeout=5):
        EventSocket.__init__(self, filter, pool_size)
        self.transport = OutboundTransport(socket, address, connect_timeout)
        self._uuid = None
        self._channel = None
        # Connects.
        self.connect()
        # Runs the main funtion .
        try:
            self.run()
        finally:
            # Disconnects.
            self.disconnect()

    def connect(self):
        # Starts event handler for this client/session.
        self.start_event_handler()

        # Sends connect and sets timeout while connecting.
        timer = Timeout(self.transport.get_connect_timeout())
        timer.start()
        try:
            connect_response = self._protocol_send("connect")
            if not connect_response.is_success():
                self.disconnect()
                raise ConnectError("Error while connecting")
        except Timeout:
            self.disconnect()
            raise ConnectError("Timeout connecting") 
        finally:
            timer.cancel()

        # Sets channel and channel unique id from this event
        self._channel = connect_response
        self._uuid = connect_response.get_header("Unique-ID")

        # Set connected flag to True
        self.connected = True

        # Sets event filter or raises ConnectError
        if self._filter:
            filter_response = self.eventplain(self._filter)
            if not filter_response.is_success():
                self.disconnect()
                raise ConnectError("Event filter failure")

    def get_channel(self):
        return self._channel

    def get_channel_unique_id(self):
        return self._uuid

    def run(self):
        '''
        This method must be implemented by subclass.

        This is the entry point for outbound application.
        '''
        pass


class OutboundServer(StreamServer):
    '''
    FreeSWITCH Outbound Event Server
    '''
    def __init__(self, address, handle_class, filter="ALL"):
        self._filter = filter
        self._handle_class = handle_class
        StreamServer.__init__(self, address, self.do_handle)

    def do_handle(self, socket, address):
        self._handle_class(socket, address, self._filter)




if __name__ == '__main__':
    outboundserver = OutboundServer(('127.0.0.1', 8084), OutboundEventSocket)
    outboundserver.serve_forever()




# -*- coding: utf-8 -*-
"""
Transport classes
"""

import gevent.socket as socket
from telephonie.core.errors import ConnectError


class Transport(object):
    def write(self, data):
        self.sockfd.write(data)
        self.sockfd.flush()

    def read_line(self):
        return self.sockfd.readline()

    def read(self, length):
        return self.sockfd.read(length)

    def close(self):
        try:
            self.sock.close()
        except:
            pass

    def get_connect_timeout(self):
        return self.timeout


class InboundTransport(Transport):
    def __init__(self, host, port, connect_timeout=5):
        self.host = host
        self.port = port
        self.timeout = connect_timeout
        self.sockfd = None

    def connect(self):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.settimeout(self.timeout)
        self.sock.connect((self.host, self.port))
        self.sock.settimeout(None)
        self.sockfd = self.sock.makefile()

    def write(self, data):
        if not self.sockfd:
            raise ConnectError('not connected')
        self.sockfd.write(data)
        self.sockfd.flush()
        


class OutboundTransport(Transport):
    def __init__(self, socket, address, connect_timeout=5):
        self.sock = socket
        self.sockfd = socket.makefile()
        self.address = address
        self.timeout = connect_timeout




# -*- coding: utf-8 -*-
"""
FreeSWITCH Commands class

Please refer to http://wiki.freeswitch.org/wiki/Mod_event_socket#Command_documentation
"""


class Commands(object):
    def api(self, args):
        "Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#api"
        return self._protocol_send("api", args)

    def bgapi(self, args):
        "Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#bgapi"
        return self._protocol_send("bgapi", args)

    def exit(self):
        "Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#exit"
        res = self._protocol_send("exit")
        try:
            self.disconnect()
        except:
            pass
        return res

    def eventplain(self, args):
        "Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#event"
        return self._protocol_send('event plain', args)

    def event(self, args):
        "Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#event"
        return self._protocol_send("event", args)

    def filter(self, args):
        """Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#filter

        The user might pass any number of values to filter an event for. But, from the point
        filter() is used, just the filtered events will come to the app - this is where this
        function differs from event().

        >>> filter('Event-Name MYEVENT')
        >>> filter('Unique-ID 4f37c5eb-1937-45c6-b808-6fba2ffadb63')
        """
        return self._protocol_send('filter', args)

    def filter_delete(self, args):
        """Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#filter_delete

        >>> filter_delete('Event-Name MYEVENT')
        """
        return self._protocol_send('filter delete', args)

    def divert_events(self, flag):
        """Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#divert_events

        >>> divert_events("off")
        >>> divert_events("on")
        """
        return self._protocol_send('divert_events', flag)

    def sendevent(self, args):
        """Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#sendevent

        >>> sendevent("CUSTOM\nEvent-Name: CUSTOM\nEvent-Subclass: myevent::test\n")

        This example will send event :
          Event-Subclass: myevent%3A%3Atest
          Command: sendevent%20CUSTOM
          Event-Name: CUSTOM
        """
        return self._protocol_send('sendevent', args)

    def auth(self, args):
        """Please refer to http://wiki.freeswitch.org/wiki/Event_Socket#auth
        
        This method is only used for Inbound connections.
        """
        return self._protocol_send("auth", args)

    def myevents(self, uuid=""):
        """For Inbound connection, please refer to http://wiki.freeswitch.org/wiki/Event_Socket#Special_Case_-_.27myevents.27

        For Outbound connection, please refer to http://wiki.freeswitch.org/wiki/Event_Socket_Outbound#Events

        >>> myevents()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_send("myevents", uuid)

    def verbose_events(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_verbose_events

        >>> verbose_events()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("verbose_events", "", uuid, lock)

    def answer(self, uuid="", lock=True):
        """
        Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_answer

        >>> answer()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("answer", "", uuid, lock)

    def bridge(self, args, uuid="", lock=True):
        """
        Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_bridge

        >>> bridge("{ignore_early_media=true}sofia/gateway/myGW/177808")
        
        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("bridge", args, uuid, lock)

    def hangup(self, cause="", uuid="", lock=True):
        """Hangup call.

        Hangup `cause` list : http://wiki.freeswitch.org/wiki/Hangup_Causes (Enumeration column)

        >>> hangup()
        
        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("hangup", cause, uuid, lock)

    def ring_ready(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_ring_ready

        >>> ring_ready()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("ring_ready", "", uuid)

    def record_session(self, filename, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_record_session
        
        >>> record_session("/tmp/dump.gsm")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("record_session", filename, uuid, lock)

    def bind_meta_app(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_bind_meta_app
        
        >>> bind_meta_app("2 ab s record_session::/tmp/dump.gsm")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("bind_meta_app", args, uuid, lock)

    def wait_for_silence(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_wait_for_silence
        
        >>> wait_for_silence("200 15 10 5000")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("wait_for_silence", args, uuid, lock)

    def sleep(self, milliseconds, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_sleep
        
        >>> sleep(5000)
        >>> sleep("5000")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("sleep", milliseconds, uuid, lock)

    def vmd(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Mod_vmd
        
        >>> vmd("start")
        >>> vmd("stop")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("vmd", args, uuid, lock)

    def set(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_set
        
        >>> set("ringback=${us-ring}")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("set", args, uuid, lock)

    def set_global(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_set_global
        
        >>> set_global("global_var=value")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("set_global", args, uuid, lock)

    def unset(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_unset
        
        >>> unset("ringback")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("unset", args, uuid, lock)

    def start_dtmf(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_start_dtmf

        >>> start_dtmf()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("start_dtmf", "", uuid, lock)

    def stop_dtmf(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_stop_dtmf

        >>> stop_dtmf()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("stop_dtmf", "", uuid, lock)

    def start_dtmf_generate(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_start_dtmf_generate

        >>> start_dtmf_generate()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("start_dtmf_generate", "true", uuid, lock)

    def stop_dtmf_generate(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_stop_dtmf_generate

        >>> stop_dtmf_generate()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("stop_dtmf_generate", "", uuid, lock)

    def queue_dtmf(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_queue_dtmf

        Enqueue each received dtmf, that'll be sent once the call is bridged.

        >>> queue_dtmf("0123456789")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("queue_dtmf", args, uuid, lock)

    def flush_dtmf(self, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_flush_dtmf

        >>> flush_dtmf()

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("flush_dtmf", "", uuid, lock)

    def play_fsv(self, filename, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Mod_fsv
        
        >>> play_fsv("/tmp/video.fsv")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("play_fsv", filename, uuid, lock)

    def record_fsv(self, filename, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Mod_fsv
        
        >>> record_fsv("/tmp/video.fsv")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("record_fsv", filename, uuid, lock)

    def playback(self, filename, terminators=None, uuid="", lock=True, loops=1):
        """Please refer to http://wiki.freeswitch.org/wiki/Mod_playback
        
        The optional argument `terminators` may contain a string with
        the characters that will terminate the playback.
        
        >>> playback("/tmp/dump.gsm", terminators="#8")
        
        In this case, the audio playback is automatically terminated 
        by pressing either '#' or '8'.

        For Inbound connection, uuid argument is mandatory.
        """
        if not terminators:
            terminators = 'none'
        self.set("playback_terminators=%s" % terminators, uuid)
        return self._protocol_sendmsg("playback", filename, uuid, lock, loops)

    def transfer(self, args, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_transfer

        >>> transfer("3222 XML default")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("transfer", args, uuid, lock)

    def att_xfer(self, url, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_att_xfer
        
        >>> att_xfer("user/1001")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("att_xfer", url, uuid, lock)

    def endless_playback(self, filename, uuid="", lock=True):
        """Please refer to http://wiki.freeswitch.org/wiki/Misc._Dialplan_Tools_endless_playback
        
        >>> endless_playback("/tmp/dump.gsm")

        For Inbound connection, uuid argument is mandatory.
        """
        return self._protocol_sendmsg("endless_playback", filename, uuid, lock)




# -*- coding: utf-8 -*-
"""
Inbound Event Socket class
"""

import gevent
from gevent.timeout import Timeout
from telephonie.core.eventsocket import EventSocket
from telephonie.core.transport import InboundTransport
from telephonie.core.errors import ConnectError


class InboundEventSocket(EventSocket):
    '''
    FreeSWITCH Inbound Event Socket
    '''
    def __init__(self, host, port, password, filter="ALL", pool_size=500, connect_timeout=5):
        EventSocket.__init__(self, filter, pool_size)
        self.password = password
        self.transport = InboundTransport(host, port, connect_timeout=connect_timeout)

    def _wait_auth_request(self):
        '''
        Waits until auth/request event is received.
        '''
        # Sets timeout to wait for auth/request
        timer = Timeout(self.transport.get_connect_timeout())
        timer.start()
        try:
            # When auth/request is received, 
            # _authRequest method in BaseEventSocket will push this event to queue
            # so we will just wait this event here.
            return self._response_queue.get()
        except Timeout:
            raise ConnectError("Timeout waiting auth/request") 
        finally:
            timer.cancel()

    def connect(self):
        '''
        Connects to mod_eventsocket, authenticates and sets event filter.

        Returns True on success or raises ConnectError exception on failure.
        '''
        # Connects transport, if connection fails, raise ConnectError
        try:
            self.transport.connect()
        except Exception, e:
            raise ConnectError("Transport failure: %s" % str(e))

        # Starts handling events
        self.start_event_handler()

        # Waits for auth/request, if timeout, raises ConnectError
        self._wait_auth_request()

        # We are ready now !
        # Authenticate or raise ConnectError
        auth_response = self.auth(self.password)
        if not auth_response.is_success():
            self.disconnect()
            raise ConnectError("Auth failure")

        # Sets event filter or raises ConnectError
        if self._filter:
            filter_response = self.eventplain(self._filter)
            if not filter_response.is_success():
                self.disconnect()
                raise ConnectError("Event filter failure")

        # Sets connected flag to True
        self.connected = True
        return True

    def serve_forever(self):
        """
        Starts waiting for events in endless loop.
        """
        while self.is_connected(): 
            gevent.sleep(0.1)





# -*- coding: utf-8 -*-
from unittest import TestCase

from telephonie.core.inboundsocket import InboundEventSocket
from telephonie.core.eventtypes import Event
from telephonie.core.errors import ConnectError
import gevent
from gevent import socket
from gevent import Timeout
from gevent.server import StreamServer


class TestClient(object):
    '''
    Client class on test inbound server side.
    '''
    def __init__(self, sock):
        self.socket = sock
        self.fd = self.socket.makefile()
        self.auth = False
        self.event_plain = False

    def send(self, msg):
        self.fd.write(msg)
        self.fd.flush()

    def recv(self):
        return self.fd.readline()

    def close(self):
        try:
            self.socket.close()
        except:
            pass


class TestEventSocketServer(object):
    '''
    Test inbound socket server.
    '''
    def __init__(self):
        self.server = StreamServer(('127.0.0.1', 18021), self.emulate)

    def start(self):
        self.server.serve_forever()

    def emulate(self, sock, address):
        client = TestClient(sock)
        client.send("Content-Type: auth/request\n\n")
        buff = ""
        # do auth (3 tries)
        for i in range(3):
            while True:
                line = client.recv()
                if not line:
                    break
                elif line == '\r\n' or line == '\n':
                    self.check_auth(client, buff)
                    buff = ""
                    break
                else:
                    buff += line
                    if buff.startswith('exit'):
                        self.disconnect(client)
                        return
            if client.auth is True:
                break
        if client.auth is False:
            self.disconnect(client)
            return

        # wait event plain ALL (3 tries)
        buff = ""
        for i in range(3):
            while True:
                line = client.recv()
                if not line:
                    break
                elif line == '\r\n' or line == '\n':
                    self.event_plain(client, buff)
                    buff = ""
                    break
                else:
                    buff += line
                    if buff.startswith('exit'):
                        self.disconnect(client)
                        return
            if client.event_plain is True:
                break
        if client.event_plain is False:
            self.disconnect(client)
            return

        # send fake heartbeat and re_schedule events to client 10 times
        for i in range(10):
            self.send_heartbeat(client)
            gevent.sleep(0.01)
            self.send_re_schedule(client)
            gevent.sleep(0.01)

        self.disconnect(client)
        return

    def disconnect(self, client):
        client.send("Content-Type: text/disconnect-notice\nContent-Length: 67\n\nDisconnected, goodbye.\nSee you at ClueCon! http://www.cluecon.com/\n\n")
        client.close()

    def send_heartbeat(self, client):
        msg = \
"""Content-Length: 630
Content-Type: text/event-plain

Event-Name: HEARTBEAT
Core-UUID: 12640749-db62-421c-beac-4863eac76510
FreeSWITCH-Hostname: vocaldev
FreeSWITCH-IPv4: 10.0.0.108
FreeSWITCH-IPv6: %3A%3A1
Event-Date-Local: 2011-01-04%2010%3A19%3A56
Event-Date-GMT: Tue,%2004%20Jan%202011%2009%3A19%3A56%20GMT
Event-Date-Timestamp: 1294132796167745
Event-Calling-File: switch_core.c
Event-Calling-Function: send_heartbeat
Event-Calling-Line-Number: 65
Event-Info: System%20Ready
Up-Time: 0%20years,%203%20days,%2017%20hours,%2043%20minutes,%2039%20seconds,%20644%20milliseconds,%2091%20microseconds
Session-Count: 0
Session-Per-Sec: 30
Session-Since-Startup: 0
Idle-CPU: 100.000000

"""
        client.send(msg)

    def send_re_schedule(self, client):
        msg = \
"""Content-Length: 491
Content-Type: text/event-plain

Event-Name: RE_SCHEDULE
Core-UUID: 12640749-db62-421c-beac-4863eac76510
FreeSWITCH-Hostname: vocaldev
FreeSWITCH-IPv4: 10.0.0.108
FreeSWITCH-IPv6: %3A%3A1
Event-Date-Local: 2011-01-04%2010%3A19%3A56
Event-Date-GMT: Tue,%2004%20Jan%202011%2009%3A19%3A56%20GMT
Event-Date-Timestamp: 1294132796167745
Event-Calling-File: switch_scheduler.c
Event-Calling-Function: switch_scheduler_execute
Event-Calling-Line-Number: 65
Task-ID: 1
Task-Desc: heartbeat
Task-Group: core
Task-Runtime: 1294132816

"""
        client.send(msg)

    def check_auth(self, client, buff):
        # auth request
        if buff.startswith('auth '):
            try:
                password = buff.split(' ')[1].strip()
                if password == 'ClueCon':
                    client.auth = True
                    client.send("Content-Type: command/reply\nReply-Text: +OK accepted\n\n")
                    return True
                raise Exception("Invalid auth password")
            except:
                client.send("Content-Type: command/reply\nReply-Text: -ERR invalid\n\n")
                return False
        client.send("Content-Type: command/reply\nReply-Text: -ERR invalid\n\n")
        return False

    def event_plain(self, client, buff):
        if buff.startswith('event plain'):
            client.event_plain = True
            client.send("Content-Type: command/reply\nReply-Text: +OK event listener enabled plain\n\n")
            return True
        return False


class TestInboundEventSocket(InboundEventSocket):
    def __init__(self, host, port, password, filter='ALL', pool_size=500, connect_timeout=5):
        InboundEventSocket.__init__(self, host, port, password, filter, pool_size, connect_timeout)
        self.heartbeat_events = []
        self.re_schedule_events = []

    def on_re_schedule(self, ev):
        self.re_schedule_events.append(ev)

    def on_heartbeat(self, ev):
        self.heartbeat_events.append(ev)

    def serve_for_test(self):
        timeout = Timeout(10)
        timeout.start()
        try:
            while self.is_connected():
                if len(self.re_schedule_events) == 10 and len(self.heartbeat_events) == 10:
                    break
                gevent.sleep(0.01)
        finally:
            timeout.cancel()


class TestInboundCase(TestCase):
    '''
    Test case for Inbound Event Socket.
    '''
    def setUp(self):
        s = TestEventSocketServer()
        self.server_proc = gevent.spawn(s.start)
        gevent.sleep(0.2)

    def tearDown(self):
        try:
            self.server_proc.kill()
        except:
            pass
        
    def test_login_failure(self):
        isock = InboundEventSocket('127.0.0.1', 23333, 'ClueCon')
        self.assertRaises(ConnectError, isock.connect)

    def test_login_success(self):
        isock = InboundEventSocket('127.0.0.1', 18021, 'ClueCon')
        try:
            self.assertTrue(isock.connect())
        except socket.error, se:
            self.fail("socket error: %s" % str(se))
        except ConnectError, e:
            self.fail("connect error: %s" % str(e))

    def test_events(self):
        isock = TestInboundEventSocket('127.0.0.1', 18021, 'ClueCon')
        try:
            self.assertTrue(isock.connect())
        except socket.error, se:
            self.fail("socket error: %s" % str(se))
        except ConnectError, e:
            self.fail("connect error: %s" % str(e))
        try:
            isock.serve_for_test()
        except Timeout, t:
            self.fail("timeout error: cannot get all events")
        self.assertEquals(len(isock.heartbeat_events), 10)
        self.assertEquals(len(isock.re_schedule_events), 10)
        for ev in isock.heartbeat_events:
            self.assertEquals(ev.get_header('Event-Name'), 'HEARTBEAT')
        for ev in isock.re_schedule_events:
            self.assertEquals(ev.get_header('Event-Name'), 'RE_SCHEDULE')




# -*- coding: utf-8 -*-
"""
Daemonize application.
"""

import os
import sys
import grp
import pwd
from subprocess import Popen
import optparse


__default_servicename__ = os.path.splitext(os.path.basename(sys.argv[0]))[0]


def daemon(user, group, path='/', pidfile='/tmp/%s.pid' % __default_servicename__, other_groups=()):
    '''
    Daemonizes current application.
    '''
    # Get uid and gid from user and group names
    uid = int(pwd.getpwnam(user)[2])
    gid = int(grp.getgrnam(group)[2])
    # Get ID of other groups
    other_groups_id = []
    for name in other_groups:
        try:
            other_groups_id.append(int(grp.getgrnam(name)[2]) )
        except:
            pass
    # First fork
    pid = os.fork()
    if not pid == 0:
        os._exit(0)
    # Creates a session and sets the process group ID
    os.setsid()
    # Second fork
    pid = os.fork()
    if not pid == 0:
        os._exit(0)
    # Change directoty
    os.chdir(path)
    # Set umask
    os.umask(0)
    # Write pidfile
    open(pidfile, 'w').write(str(os.getpid()))
    # Set group and groups
    os.setgid(gid)
    if other_groups_id:
        os.setgroups(other_groups_id)
    # Set user
    os.setuid(uid)
    # Redirect stdout/stderr to /dev/null
    sys.stdout = sys.stderr = open(os.devnull, 'a+')


def daemon_script(script, user, group, path='/', pidfile=None, script_args=(), other_groups=(), python_bin=None):
    '''
    Daemonize a python script.
    '''
    # Autocreate path for pidfile (based on script arg) if not set
    if not pidfile:
        pidfile = '/tmp/' + os.path.splitext(os.path.basename(script))[0] + '.pid'
    # Get full/real path to script
    real_script = os.path.realpath(script)
    # Get uid and gid from user and group names
    uid = int(pwd.getpwnam(user)[2])
    gid = int(grp.getgrnam(group)[2])
    # Get ID of other groups
    other_groups_id = []
    for name in other_groups:
        try:
            other_groups_id.append(int(grp.getgrnam(name)[2]) )
        except:
            pass
    # First fork
    pid = os.fork()
    if not pid == 0:
        os._exit(0)
    # Creates a session and sets the process group ID
    os.setsid()
    # Second fork
    pid = os.fork()
    if not pid == 0:
        os._exit(0)
    # Change directoty
    os.chdir(path)
    # Set umask
    os.umask(0)
    # Set group and groups
    os.setgid(gid)
    if other_groups_id:
        os.setgroups(other_groups_id)
    # Set user
    os.setuid(uid)
    # Set python binary
    if not python_bin:
        cmd = ["/usr/bin/env", "python"]
    else:
        cmd = [python_bin]
    cmd.append(real_script)
    # Add script_args
    for arg in script_args:
        cmd.append(arg)
    # Run script
    pid = Popen(cmd).pid
    # Write pidfile
    open(pidfile, 'w').write(str(pid))
    # Redirect stdout/stderr to /dev/null
    sys.stdout = sys.stderr = open(os.devnull, 'a+')
    # Wait pid end
    os.waitpid(pid, 0)


def main():
    parser = optparse.OptionParser()
    parser.add_option("-s", "--script", action="store", type="string",
                      dest="script", help="python script SCRIPT to run (argument is mandatory)", 
                      metavar="SCRIPT")
    parser.add_option("-p", "--pidfile", action="store", type="string",
                      dest="pidfile", help="write pid to PIDFILE (argument is mandatory)",
                      metavar="PIDFILE")
    parser.add_option("-u", "--user", action="store", type="string",
                      dest="user", help="set uid to USER (argument is mandatory)", 
                      metavar="USER")
    parser.add_option("-g", "--group", action="store", type="string",
                      dest="group", help="set gid to GROUP (argument is mandatory)", 
                      metavar="GROUP")
    parser.add_option("-G", "--groups", action="append", type="string", default=[],
                      dest="groups", help="set other groups gid to OTHERGROUP (can be added multiple times)", 
                      metavar="OTHERGROUP")
    parser.add_option("-P", "--pybin", action="store", type="string", default=None,
                      dest="pybin", help="set python binary PYBIN to run script", 
                      metavar="PYBIN")
    parser.add_option("-a", "--scriptarg", action="append", type="string", default=[],
                      dest="scriptargs", help="add ARG to python script (can be added multiple times)", 
                      metavar="ARG")
    (options, args) = parser.parse_args()

    script = options.script
    user = options.user
    group = options.group
    pidfile = options.pidfile 
    ogroups = options.groups
    pybin = options.pybin
    scriptargs = options.scriptargs

    if not script or not user or not group or not pidfile:
        parser.print_help()
        sys.exit(1)

    daemon_script(script, user, group, pidfile=pidfile, 
                  script_args=scriptargs, other_groups=ogroups, 
                  python_bin=pybin)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
try:
    from setuptools import find_packages
except:
    from distutils.core import find_packages

version = open('VERSION.txt').read().strip()
author = "Telephonie Team"
author_email = "telephonie@miglu.com"
maintainer = "Telephonie Team"
maintainer_email = "telephonie@miglu.com"
licence = "MPL 1.1"

setup_args = {
      'name':'telephonie',
      'version':version,
      'description':'Telephonie framework',
      'url':'http://github.com/miglu/Telephonie',
      'author':author,
      'author_email':author_email,
      'maintainer':maintainer,
      'maintainer_email':maintainer_email,
      'platforms':['linux'],
      'long_description':'Framework to create telephony applications using FreeSWITCH',
      'packages':find_packages('src'),
      'package_dir':{'': 'src'},
      'scripts':['src/scripts/telephonied'],
      'keywords':"telecom voip telephony freeswitch ivr",
      'license':licence,
      'zip_safe':False,
      'classifiers':[
        "Programming Language :: Python",
        "Operating System :: POSIX",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications",
        "Topic :: Multimedia",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Programming Language :: Python",
        "Intended Audience :: Developers",
        "Intended Audience :: Telecommunications Industry",
        "License :: OSI Approved :: Mozilla Public License 1.1 (MPL 1.1)",
        "Development Status :: 4 - Beta"]
}


try:
    from setuptools import setup
    setup_args['install_requires'] = ['gevent']
except ImportError:
    from distutils.core import setup
    setup_args['requires'] = ['gevent']
  
setup(**setup_args)



# -*- coding: utf-8 -*-
"""
Core for Telephonie Application Framework
"""

import gevent.monkey
gevent.monkey.patch_all()


__all__ = ['transport', 
           'eventsocket', 
           'commands', 
           'eventtypes', 
           'errors', 
           'inboundsocket',
           'outboundsocket']
